#include "gbezier.h"

namespace GraphicsObjects {

    GBezier::GBezier(std::string name, std::list<Point*> points, Color * color) : GCurve(name, points, color)
    {
    }

    GBezier * GBezier::copy() {
        std::list<Point*> pointsCopy;
        std::list<Point*>::iterator pointsIt;
        for (pointsIt = points.begin(); pointsIt != points.end(); pointsIt++) {
            pointsCopy.push_back((*pointsIt)->copy());
        }
        return new GBezier(name, pointsCopy, color);
    }

    std::string GBezier::type(){
        return "Bezier";
    }

    std::list<Point*>  GBezier::generatePoints(float desiredAccuracy){
        std::list<Point*> ret;
        std::list<Point*>::iterator it = points.begin();
        Point * p1;
        Point * p2;
        Point * p3;
        Point * p4;

        while(it != points.end()){
            p1 = *it++;
            p2 = *it++;
            p3 = *it++;
            p4 = *it++;
            if (it != points.end())
                it--;

            //prepare matrixes
            float gbs[4][4];
            p1->toVector(gbs[0]);
            p2->toVector(gbs[1]);
            p3->toVector(gbs[2]);
            p4->toVector(gbs[3]);
            float mbs[4][4] = {{-1.,  3., -3., 1.},
                               { 3., -6.,  3., 0.},
                               {-3.,  3.,  0., 0.},
                               { 1.,  0.,  0., 0.}};

            //determine step based on desired accuracy
            float accuracy = INFINITY;
            float step = MAXIMUM_STEP * 2;
            while (accuracy > desiredAccuracy && step > MINIMUM_STEP) {
                step = step/2;
                Point * tmp = new Point();
                float tVector[4] = {step*step*step, step*step, step, 1};
                float tempVector[4];
                Transformations::multiply(tVector, mbs, tempVector);
                float coordinates[4];
                Transformations::multiply(tempVector, gbs, coordinates);
                tmp->fromVector(coordinates);
                accuracy = p1->distance(tmp);
            }
            if (step < MINIMUM_STEP)
                step = MINIMUM_STEP;

            for (float t = 0; t < 1; t += step) {
                Point * tmp = new Point();
                float tVector[4] = {t*t*t, t*t, t, 1};
                float tempVector[4];
                Transformations::multiply(tVector, mbs, tempVector);
                float coordinates[4];
                Transformations::multiply(tempVector, gbs, coordinates);
                tmp->fromVector(coordinates);

                ret.push_back(tmp);
            }

            //t = 1
            Point * tmp = new Point();
            float tVector[4] = {1, 1, 1, 1};
            float tempVector[4];
            Transformations::multiply(tVector, mbs, tempVector);
            float coordinates[4];
            Transformations::multiply(tempVector, gbs, coordinates);
            tmp->fromVector(coordinates);

            ret.push_back(tmp);
        }
        return ret;
    }

} // namespace GraphicsObjects
