#include "gbspline.h"

namespace GraphicsObjects {

    GBSpline::GBSpline(std::string name, std::list<Point*> points, Color * color) : GCurve(name, points, color)
    {
    }

    GBSpline * GBSpline::copy() {
        std::list<Point*> pointsCopy;
        std::list<Point*>::iterator pointsIt;
        for (pointsIt = points.begin(); pointsIt != points.end(); pointsIt++) {
            pointsCopy.push_back((*pointsIt)->copy());
        }
        return new GBSpline(name, pointsCopy, color);
    }

    std::string GBSpline::type(){
        return "B-Spline";
    }

    std::list<Point*>  GBSpline::generatePoints(float desiredAccuracy){
        std::list<Point*> generatedPoints;
        std::list<Point*>::iterator it = points.begin();
        Point * p1;
        Point * p2;
        Point * p3;
        Point * p4;

        p1 = *it++;
        p2 = *it++;
        p3 = *it++;


        while(it != points.end()){
            p4 = *it++;

            //prepare matrixes
            float gbs[4][4];
            p1->toVector(gbs[0]);
            p2->toVector(gbs[1]);
            p3->toVector(gbs[2]);
            p4->toVector(gbs[3]);
            float gbsTranspose[4][4];
            Transformations::transpose(gbs, gbsTranspose);
            float mbs[4][4] = {{-1./6,  1./2, -1./2, 1./6},
                               { 1./2, -1.,    1./2, 0.  },
                               {-1./2,  0.,    1./2, 0.  },
                               { 1./6,  2./3,  1./6, 0.  }};

            //determine step based on desired accuracy
            float beziermbs[4][4] = {{-1.,  3., -3., 1.},
                                     { 3., -6.,  3., 0.},
                                     {-3.,  3.,  0., 0.},
                                     { 1.,  0.,  0., 0.}};
            float accuracy = INFINITY;
            float step = MAXIMUM_STEP * 2;
            while (accuracy > desiredAccuracy && step > MINIMUM_STEP) {
                step = step/2;
                Point * tmp = new Point();
                float tVector[4] = {step*step*step, step*step, step, 1};
                float tempVector[4];
                Transformations::multiply(tVector, beziermbs, tempVector);
                float coordinates[4];
                Transformations::multiply(tempVector, gbs, coordinates);
                tmp->fromVector(coordinates);
                accuracy = p1->distance(tmp);
            }
            if (step < MINIMUM_STEP)
                step = MINIMUM_STEP;

            int n = 1 / step;

            //calculate initial values
            float c[4][4];
            float f[3], df[3], ddf[3], dddf[3];
            for (int i = 0; i < 3; i++) {
                Transformations::multiply(mbs, gbsTranspose[i], c[i]);
                f[i] = c[i][3];
                df[i] = c[i][0]*step*step*step + c[i][1]*step*step + c[i][2]*step;
                ddf[i] = 6*c[i][0]*step*step*step + 2*c[i][1]*step*step;
                dddf[i] = 6*c[i][0]*step*step*step;
            }

            for (int i = 0; i < n; i++){
                Point * tmp = new Point();

                tmp->fromVector(f);
                for (int i = 0; i < 3; i++) {
                    f[i] += df[i];
                    df[i] += ddf[i];
                    ddf[i] += dddf[i];
                }

                generatedPoints.push_back(tmp);
            }

            Point * tmp = new Point();
            tmp->x = (p2->x + 4*p3->x + p4->x)/6;
            tmp->y = (p2->y + 4*p3->y + p4->y)/6;
            tmp->z = (p2->z + 4*p3->z + p4->z)/6;

            generatedPoints.push_back(tmp);

            p1 = p2;
            p2 = p3;
            p3 = p4;
        }
        return generatedPoints;
    }

} // namespace GraphicsObjects
