#include "gobject.h"
#define RIGHT 0
#define LEFT 1
#define TOP 2
#define BOTTOM 3
#define FRONT 4
#define BACK 5

namespace GraphicsObjects {

    GObject::GObject(){
        GObject::name = "";
        color = new Color(0, 0, 0);
    }

    GObject::GObject(std::string name){
        GObject::name = name;
        color = new Color(0, 0, 0);
    }

    GObject::GObject(std::string name, Color * color){
        GObject::name = name;
        GObject::color = color;
    }

    GObject::~GObject(){
        delete this;
    }

    void GObject::add(float x, float y, float z){
        Point * p = new Point(x, y, z);
        points.push_back(p);
    }

    void GObject::add(Point * p){
        points.push_back(p);
    }

    Point * GObject::center(){
        DBG3("initializing the method center() of object [",name,"]");
        float cx = 0;
        float cy = 0;
        float cz = 0;
        std::list<Point*>::iterator it;
        for (it = points.begin(); it != points.end(); it++){
            cx += (*it)->x;
            cy += (*it)->y;
            cz += (*it)->z;
        }
        if(!points.empty()){
            cx = cx/points.size();
            cy = cy/points.size();
            cz = cz/points.size();
        }
        return new Point(cx, cy, cz);
    }

    std::list<Point*> GObject::getPoints(){
        return points;
    }
    std::list<Point *> GObject::clipLine(Point *p1, Point *p2){
        std::list<Point *> clippedPoints;
        bool rc1[6];
        bool rc2[6];

        float x1 = p1->x;
        float y1 = p1->y;
        float z1 = p1->z;
        float x2 = p2->x;
        float y2 = p2->y;
        float z2 = p2->z;

        float xTop, xBottom, xFront, xBack, yRight, yLeft, yFront, yBack, zRight, zLeft, zTop, zBottom;
        float dx = (x2 - x1);
        float dy = (y2 - y1);
        float dz = (z2 - z1);

        rc1[LEFT] = x1 < -1;
        rc1[RIGHT] = x1 > +1;
        rc1[BOTTOM] = y1 < -1;
        rc1[TOP] = y1 > +1;
        rc1[BACK] = z1 < -1;
        rc1[FRONT] = z1 > +1;

        rc2[LEFT] = x2 < -1;
        rc2[RIGHT] = x2 > +1;
        rc2[BOTTOM] = y2 < -1;
        rc2[TOP] = y2 > +1;
        rc2[BACK] = z2 < -1;
        rc2[FRONT] = z2 > +1;

        //se reta dentro
        if((!rc1[TOP])&&(!rc1[BOTTOM])&&(!rc1[RIGHT])&&(!rc1[LEFT])&&(!rc1[FRONT])&&(!rc1[BACK])&&
           (!rc2[TOP])&&(!rc2[BOTTOM])&&(!rc2[RIGHT])&&(!rc2[LEFT])&&(!rc2[FRONT])&&(!rc2[BACK])) {
            clippedPoints.push_back(p1);
            clippedPoints.push_back(p2);
            return clippedPoints;
        }
        //se reta fora
        if((rc1[TOP]&&rc2[TOP])||(rc1[BOTTOM]&&rc2[BOTTOM])||
           (rc1[RIGHT]&&rc2[RIGHT])||(rc1[LEFT]&&rc2[LEFT])||
           (rc1[FRONT]&&rc2[FRONT])||(rc1[BACK]&&rc2[BACK])) {
            return clippedPoints;
        }

        //se ponto 1 dentro
        bool p1in = !(rc1[TOP]||rc1[BOTTOM]||rc1[RIGHT]||rc1[LEFT]||rc1[FRONT]||rc1[BACK]);
        if(p1in)
            clippedPoints.push_back(p1);

        //se reta cruza pela direita
        if(rc1[RIGHT]||rc2[RIGHT]){
            yRight = dy/dx * (1 - x1) + y1;
            zRight = dz/dx * (1 - x1) + z1;
            if((yRight >= -1 && yRight <= 1) && (zRight >= -1 && zRight <= 1))
                clippedPoints.push_back(new Point(1, yRight,zRight));
        }

        //se reta cruza pela esquerda
        if(rc1[LEFT]||rc2[LEFT]){
            yLeft = dy/dx * (-1 - x1) + y1;
            zLeft = dz/dx * (-1 - x1) + z1;
            if((yLeft >= -1 && yLeft <= 1) && (zLeft >= -1 && zLeft <= 1))
                clippedPoints.push_back(new Point(-1, yLeft, zLeft));
        }

        //evita que sejam adicionados mais pontos caso a reta passe por um canto
        bool p2in = !(rc2[TOP]||rc2[BOTTOM]||rc2[RIGHT]||rc2[LEFT]||rc2[FRONT]||rc2[BACK]);
        if(p2in)
            clippedPoints.push_back(p2);

        if (clippedPoints.size() == 2)
            return clippedPoints;

        if(p2in)
            clippedPoints.pop_back();

        //se reta cruza por cima
        if(rc1[TOP]||rc2[TOP]){
            xTop = dx/dy * (1 - y1) + x1;
            zTop = dz/dy * (1 - y1) + z1;
            if((xTop >= -1 && xTop <= 1) && (zTop >= -1 && zTop <= 1))
                clippedPoints.push_back(new Point(xTop, 1, zTop));
        }

        //se reta cruza por baixo
        if(rc1[BOTTOM]||rc2[BOTTOM]){
            xBottom = dx/dy * (-1 - y1) + x1;
            zBottom = dz/dy * (-1 - y1) + z1;
            if((xBottom >= -1 && xBottom <= 1) && (zBottom >= -1 && zBottom <= 1))
                clippedPoints.push_back(new Point(xBottom, -1, zBottom));
        }

        //evita que sejam adicionados mais pontos caso a reta passe por um canto
        if(p2in)
            clippedPoints.push_back(p2);

        if (clippedPoints.size() == 2)
            return clippedPoints;

        if(p2in)
            clippedPoints.pop_back();

        //se reta cruza pela frente
        if(rc1[FRONT]||rc2[FRONT]){
            xFront = dx/dz * (1 - z1) + x1;
            yFront = dy/dz * (1 - z1) + y1;
            if((xFront >= -1 && xFront <= 1) && (yFront >= -1 && yFront <= 1))
                clippedPoints.push_back(new Point(xFront, yFront, 1));
        }

        //se reta cruza por trs
        if(rc1[BACK]||rc2[BACK]){
            xBack = dx/dz * (-1 - z1) + x1;
            yBack = dy/dz * (-1 - z1) + y1;
            if((xBack >= -1 && xBack <= 1) && (yBack >= -1 && yBack <= 1))
                clippedPoints.push_back(new Point(xBack, yBack, -1));
        }

        if(p2in)
            clippedPoints.push_back(p2);

        return clippedPoints;
    }


    std::string GObject::toString(){
        std::string ret = name+"("+ type()+")";
        return ret;
    }

    std::string GObject::toStringOfPoints(){
        std::list<Point*>::iterator it;
        std::string ret="["+toString()+"]->[";
        for (it=points.begin();it!=points.end();it++){
            float xval=(*it)->x;
            float yval=(*it)->y;
            float zval=(*it)->z;
            std::stringstream sstrx;
            std::stringstream sstry;
            std::stringstream sstrz;
            sstrx << xval;
            std::string valx = sstrx.str();
            sstry << yval;
            std::string valy = sstry.str();
            sstrz << zval;
            std::string valz = sstry.str();
            ret+="("+valx+","+valy+","+valz+")";

        }
        ret=ret+"]";
        return ret;
    }

} // namespace GraphicsObjects
