#include "gwireframe.h"

namespace GraphicsObjects {

    GWireframe::GWireframe(std::string name, std::list<Edge*> edges, Color * color): GObject(name, color)
    {
        GWireframe::edges = edges;

        std::list<Edge*>::iterator edgesIt;
        for (edgesIt = edges.begin(); edgesIt != edges.end(); edgesIt++) {
            points.push_back((*edgesIt)->p1);
            points.push_back((*edgesIt)->p2);
        }
        points.sort();
        points.unique();
    }

    GWireframe * GWireframe::copy(){
        std::list<Edge*> edgesCopy;
        std::list<Edge*>::iterator edgesIt;
        for (edgesIt = edges.begin(); edgesIt != edges.end(); edgesIt++) {
            edgesCopy.push_back((*edgesIt)->copy());
        }

        return new GWireframe(name, edgesCopy, color);
    };

    void GWireframe::addEdge(Point * p1, Point * p2){
        Edge * edge = new Edge(p1, p2);
        edges.push_back(edge);
    };

    std::string GWireframe::type(){
        return "Wireframe";
    };

    bool GWireframe::clip(){
        points.clear();
        std::list<Edge*>::iterator it;
        for(it = edges.begin(); it != edges.end(); it++) {
            std::list<Point*> clippedLine = clipLine((*it)->p1, (*it)->p2);
            points.splice(points.end(), clippedLine);
        }
        return points.size() > 0;
    };

} // namespace GraphicsObjects
