#include "gui.h"
#include "ui_gui.h"

gui::gui(QWidget *parent) :
        QMainWindow(parent),
        ui(new Ui::gui),
        scene(new QGraphicsScene(this))
{
    world = new World();
    window = new Window(world, 100, 100, 200, 100);

    ui->setupUi(this);
    ui->Canvas->setScene(scene);


    ui->PolygonPoints->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    ui->BezierPoints->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    ui->BSplinePoints->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    ui->WireframeVertices->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    ui->WireframeEdges->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    ui->CurvedSurfacePointsTable->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
    ui->CurvedSurfacePointsTable->verticalHeader()->setResizeMode(QHeaderView::Stretch);

    redraw();
}

gui::~gui()
{
    delete ui;
    delete scene;
}

void gui::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void gui::resizeEvent(QResizeEvent *e)
{
    redraw();
}

void gui::redraw(){
    ui->Canvas->scene()->clear();
    ui->Objectlist->clear();
    scene->setSceneRect(0, 0, ui->Canvas->width(), ui->Canvas->height());
    QPen pen = *new QPen(*new QColor(255,0,0,255));

    std::list<GObject*> vpObjects = window->getViewportObjectsClipped(new Point(10,10,0), new Point(ui->Canvas->width()-13,ui->Canvas->height()-13,0), 20);
    std::list<GObject*>::iterator it;

    for (it = vpObjects.begin(); it != vpObjects.end(); it++){
        GObject * obj = (*it);
        Color * color = obj->color;
        QPen pen = *new QPen(*new QColor(color->red(),color->green(),color->blue(),255));

        QBrush brush = *new QBrush(*new QColor(color->red(),color->green(),color->blue(),255));
        QVector<QPointF> pointsf;

        ui->Objectlist->addItem(obj->toString().c_str());

        std::list<Point*> points = obj->getPoints();
        std::list<Point*>::iterator itp = points.begin();
        Point * p1;

        while(itp!=points.end()){
            p1 = *itp++;
            pointsf.append(*new QPointF(p1->x, p1->y));
        }

        if (obj->type().compare("Point") == 0) {
            scene->addLine(*new QLineF(pointsf.at(0),pointsf.at(0)),pen);

        } else if (obj->type().compare("Line") == 0){
            scene->addLine(*new QLineF(pointsf.at(0),pointsf.at(1)), pen);

        } else if (obj->type().compare("Polygon") == 0){
            GPolygon * pol = static_cast<GPolygon*> (obj);
            if (pol->fill)
                scene->addPolygon(*new QPolygonF(pointsf), pen, brush);
            else
                scene->addPolygon(*new QPolygonF(pointsf), pen);

        } else if (obj->type().compare("Bezier") == 0){
            int size = pointsf.size()-1;
            for (int i=0; i<size; i++){
                scene->addLine(*new QLineF(pointsf.at(i),pointsf.at(i+1)), pen);
                i++;
            }

        } else if (obj->type().compare("B-Spline") == 0){
            int size = pointsf.size()-1;
            for (int i=0; i<size; i++){
                scene->addLine(*new QLineF(pointsf.at(i),pointsf.at(i+1)), pen);
                i++;
            }

        } else if (obj->type().compare("Wireframe") == 0) {
            int size = pointsf.size()-1;
            for (int i=0; i<size; i++){
                scene->addLine(*new QLineF(pointsf.at(i),pointsf.at(i+1)), pen);
                i++;
            }

        } else if (obj->type().compare("B-Spline Surface") == 0){
            int size = pointsf.size()-1;
            for (int i=0; i<size; i++){
                scene->addLine(*new QLineF(pointsf.at(i),pointsf.at(i+1)), pen);
                i++;
            }

        }

    }

    scene->addRect(10, 10, ui->Canvas->width()-23, ui->Canvas->height()-23, pen, Qt::NoBrush);

    std::stringstream centerStream;
    centerStream << window->center->point()->x << "," << window->center->point()->y << "," << window->center->point()->z;
    std::string center = centerStream.str();
    ui->WindowCenter->setText(center.c_str());

    std::stringstream projectionStream;
    projectionStream << window->distanceFromProjectionCenter;
    std::string projectionCenterDistance = projectionStream.str();
    ui->WindowProjectionCenterDistance->setText(projectionCenterDistance.c_str());

    std::stringstream vpnStream;
    vpnStream << window->vpn->point()->x << "," << window->vpn->point()->y << "," << window->vpn->point()->z;
    std::string vpn = vpnStream.str();
    ui->WindowVPN->setText(vpn.c_str());

    std::stringstream vpuStream;
    vpuStream << window->vpu->point()->x << "," << window->vpu->point()->y << "," << window->vpu->point()->z;
    std::string vpu = vpuStream.str();
    ui->WindowVPU->setText(vpu.c_str());

    std::stringstream widthStream;
    widthStream << window->width;
    std::string width = projectionStream.str();
    ui->WindowWidth->setText(width.c_str());

    std::stringstream heightStream;
    heightStream << window->height;
    std::string height = projectionStream.str();
    ui->WindowHeight->setText(height.c_str());

    std::stringstream depthStream;
    depthStream << window->depth;
    std::string depth = projectionStream.str();
    ui->WindowDepth->setText(depth.c_str());
}

void gui::on_PolygonAdd_clicked()
{
    if (ui->PolygonPoints->rowCount() > 2){
        std::list<Point*> points;

        while (ui->PolygonPoints->rowCount() > 0) {
            float x = atof(ui->PolygonPoints->item(0,0)->text());
            float y = atof(ui->PolygonPoints->item(0,1)->text());
            float z = atof(ui->PolygonPoints->item(0,2)->text());

            points.push_back(new Point(x, y, z));

            ui->PolygonPoints->removeRow(0);
        }

        std::string name = ui->PolygonName->text().latin1();
        bool fill = ui->fillCheckBox->isChecked();
        world->createPolygon(name, points, new Color(), fill);

        redraw();
    }
}

void gui::on_LineAdd_clicked()
{
    float LineX1 = atof(ui->LineX1->text());
    float LineY1 = atof(ui->LineY1->text());
    float LineZ1 = atof(ui->LineZ1->text());

    float LineX2 = atof(ui->LineX2->text());
    float LineY2 = atof(ui->LineY2->text());
    float LineZ2 = atof(ui->LineZ2->text());

    std::string name = ui->LineName->text().latin1();
    world->createLine(name, new Point(LineX1, LineY1, LineZ1), new Point(LineX2, LineY2, LineZ2), new Color());

    redraw();
}

void gui::on_PointAdd_clicked()
{
    std::string name = ui->PointName->text().latin1();
    float x = atof(ui->PointX->text());
    float y = atof(ui->PointY->text());
    float z = atof(ui->PointZ->text());
    world->createPoint(name, new Point(x, y, z), new Color());

    redraw();
}

void gui::on_Navigateup_clicked()
{
    float rateNavigate = atof(ui->Navigaterate->text())/100;
    window->navigate(Y_AXIS, rateNavigate);
    redraw();
}

void gui::on_Navigatedown_clicked()
{
    float rateNavigate = atof(ui->Navigaterate->text())/100;
    window->navigate(Y_AXIS, -rateNavigate);
    redraw();

}

void gui::on_Navigateright_clicked()
{
    float rateNavigate = atof(ui->Navigaterate->text())/100;
    window->navigate(X_AXIS, rateNavigate);
    redraw();
}

void gui::on_Navigateleft_clicked()
{
    float rateNavigate = atof(ui->Navigaterate->text())/100;
    window->navigate(X_AXIS, -rateNavigate);
    redraw();
}

void gui::on_Navigatefront_clicked()
{
    float rateNavigate = atof(ui->Navigaterate->text())/100;
    window->navigate(Z_AXIS, rateNavigate);
    redraw();
}

void gui::on_Navigateback_clicked()
{
    float rateNavigate = atof(ui->Navigaterate->text())/100;
    window->navigate(Z_AXIS, -rateNavigate);
    redraw();
}

void gui::on_Zoomin_clicked()
{
    if (atof(ui->Zoomrate->text()) >= 100) {
        ui->Zoomrate->setText("99");
    }
    window->zoom(atof(ui->Zoomrate->text())/100);
    redraw();
}

void gui::on_Zoomout_clicked()
{
    window->zoom(-atof(ui->Zoomrate->text())/100);
    redraw();
}

void gui::on_Rotatewindow_clicked()
{
    if (ui->WindowRotationAxisX->isChecked())
        window->rotate(X_AXIS, atof(ui->Rotatewindowangle->text()));
    else if (ui->WindowRotationAxisY->isChecked())
        window->rotate(Y_AXIS, atof(ui->Rotatewindowangle->text()));
    else if (ui->WindowRotationAxisZ->isChecked())
        window->rotate(Z_AXIS, atof(ui->Rotatewindowangle->text()));
    redraw();
}

void gui::on_PolygonAddPoint_clicked()
{
    if(ui->PolygonX->text().compare("")!=0&&ui->PolygonY->text().compare("")!=0&&ui->PolygonZ->text().compare("")!=0){
        ui->PolygonPoints->insertRow(ui->PolygonPoints->rowCount());

        QTableWidgetItem *tmpX = new QTableWidgetItem();
        tmpX->setText(ui->PolygonX->text());
        ui->PolygonPoints->setItem(ui->PolygonPoints->rowCount()-1, 0, tmpX);

        QTableWidgetItem *tmpY = new QTableWidgetItem();
        tmpY->setText(ui->PolygonY->text());
        ui->PolygonPoints->setItem(ui->PolygonPoints->rowCount()-1, 1, tmpY);

        QTableWidgetItem *tmpZ = new QTableWidgetItem();
        tmpZ->setText(ui->PolygonZ->text());
        ui->PolygonPoints->setItem(ui->PolygonPoints->rowCount()-1, 2, tmpZ);

        ui->PolygonX->setText("0");
        ui->PolygonY->setText("0");
        ui->PolygonZ->setText("0");
    }
}


void gui::on_Rotate_clicked()
{
    float angle = atof(ui->Rotateangle->text());
    int index = ui->Objectlist->currentIndex();

    std::list<GObject*>::iterator it = world->objects.begin();
    if(index >= 0) {
        for (int i=0; i<index; i++)
            it++;
        GObject * obj = (*it); //pega o objeto selecionado no combobox

        Point * axisVector;
        if (ui->ObjectRotationAxisX->isChecked()) {
            axisVector = new Point(1, 0, 0);
        } else if (ui->ObjectRotationAxisY->isChecked()) {
            axisVector = new Point(0, 1, 0);
        } else if (ui->ObjectRotationAxisZ->isChecked()) {
            axisVector = new Point(0, 0, 1);
        } else if (ui->ObjectRotationAxisArbitrary->isChecked()) {
            axisVector = new Point(atof(ui->Axisvectorx->text()), atof(ui->Axisvectory->text()), atof(ui->Axisvectorz->text()));
        }

        if (ui->ObjectRotationPointCenter->isChecked()) {
            world->rotateAroundCenter(obj->name, axisVector, angle);
        } else if (ui->ObjectRotationPointOrigin->isChecked()) {
            world->rotateAroundOrigin(obj->name, axisVector, angle);
        } else if (ui->ObjectRotationPointArbitrary->isChecked()) {
            Point * anchor = new Point(atof(ui->Rotateanchorx->text()), atof(ui->Rotateanchory->text()), atof(ui->Rotateanchorz->text()));
            world->rotate(obj->name, anchor, axisVector, angle);
        }

        redraw();
    }

}

void gui::on_Scale_clicked()
{
    int index = ui->Objectlist->currentIndex();
    std::list<GObject*>::iterator it = world->objects.begin();
    if(index >= 0){
        for (int i=0; i<index; i++)
            it++;
        GObject * obj = (*it); //pega o objeto selecionado no combobox

        world->scale(obj->name,atof(ui->Scalingxrate->text()), atof(ui->Scalingyrate->text()), atof(ui->Scalingzrate->text()));

        redraw();
    }
    else{
        //no existe objeto no mundo
    }

}

void gui::on_Translation_clicked()
{
    int index = ui->Objectlist->currentIndex();
    std::list<GObject*>::iterator it = world->objects.begin();
    if(index >= 0){
        for (int i=0; i<index; i++)
            it++;
        GObject * obj = (*it); //pega o objeto selecionado no combobox
        world->translate(obj->name,atof(ui->Translationdx->text()),atof(ui->Translationdy->text()),atof(ui->Translationdz->text()));
        redraw();
    }
    else{
        //no existe objeto no mundo
    }

}

void gui::on_Changecolor_clicked()
{
    int index = ui->Objectlist->currentIndex();
    std::list<GObject*>::iterator it;
    it=world->objects.begin();
    if(index>=0){
        for (int i=0;i<index;i++)
            it++;
        GObject * obj = (*it); //pega o objeto selecionado no combobox
        if (obj->type().compare("Polygon")==0){
            GPolygon * pol = static_cast< GPolygon *>(obj);
            pol->fill = ui->fillCheckBox->isChecked();
        }
        obj->color->setRGB(atof(ui->Rcolor->text()),atof(ui->Gcolor->text()),atof(ui->Bcolor->text()));

        redraw();
    }
    else{
        //no existe objeto no mundo
    }

}

void gui::on_BezierAddPoint_clicked()
{

    if(ui->BezierX->text().compare("")!=0&&ui->BezierY->text().compare("")!=0&&ui->BezierZ->text().compare("")!=0){
        ui->BezierPoints->insertRow(ui->BezierPoints->rowCount());

        QTableWidgetItem *tmpX = new QTableWidgetItem();
        tmpX->setText(ui->BezierX->text());
        ui->BezierPoints->setItem(ui->BezierPoints->rowCount()-1, 0, tmpX);

        QTableWidgetItem *tmpY = new QTableWidgetItem();
        tmpY->setText(ui->BezierY->text());
        ui->BezierPoints->setItem(ui->BezierPoints->rowCount()-1, 1, tmpY);

        QTableWidgetItem *tmpZ = new QTableWidgetItem();
        tmpZ->setText(ui->BezierZ->text());
        ui->BezierPoints->setItem(ui->BezierPoints->rowCount()-1, 2, tmpZ);

        ui->BezierX->setText("0");
        ui->BezierY->setText("0");
        ui->BezierZ->setText("0");

        ui->BezierAdd->setEnabled((ui->BezierPoints->rowCount()-1) % 3 == 0);
    }
}

void gui::on_BSplineAddPoint_clicked()
{

    if(ui->BSplineX->text().compare("")!=0&&ui->BSplineY->text().compare("")!=0&&ui->BSplineZ->text().compare("")!=0){
        ui->BSplinePoints->insertRow(ui->BSplinePoints->rowCount());

        QTableWidgetItem *tmpX = new QTableWidgetItem();
        tmpX->setText(ui->BSplineX->text());
        ui->BSplinePoints->setItem(ui->BSplinePoints->rowCount()-1, 0, tmpX);

        QTableWidgetItem *tmpY = new QTableWidgetItem();
        tmpY->setText(ui->BSplineY->text());
        ui->BSplinePoints->setItem(ui->BSplinePoints->rowCount()-1, 1, tmpY);

        QTableWidgetItem *tmpZ = new QTableWidgetItem();
        tmpZ->setText(ui->BSplineZ->text());
        ui->BSplinePoints->setItem(ui->BSplinePoints->rowCount()-1, 2, tmpZ);

        ui->BSplineX->setText("0");
        ui->BSplineY->setText("0");
        ui->BSplineZ->setText("0");

        ui->BSplineAdd->setEnabled(ui->BSplinePoints->rowCount() >= 4);
    }

}

void gui::on_BezierAdd_clicked()
{
    if (ui->BezierPoints->rowCount() > 3){
        std::list<Point*> points;

        while (ui->BezierPoints->rowCount() > 0) {
            float x = atof(ui->BezierPoints->item(0, 0)->text());
            float y = atof(ui->BezierPoints->item(0, 1)->text());
            float z = atof(ui->BezierPoints->item(0, 2)->text());
            points.push_back(new Point(x, y, z));

            ui->BezierPoints->removeRow(0);
        }

        std::string name = ui->BezierName->text().latin1();
        world->createBezierCurve(name, points, new Color());

        redraw();
    }

}

void gui::on_BSplineAdd_clicked()
{
    if (ui->BSplinePoints->rowCount() > 3){
        std::list<Point*> points;

        while (ui->BSplinePoints->rowCount() > 0) {
            float x = atof(ui->BSplinePoints->item(0,0)->text());
            float y = atof(ui->BSplinePoints->item(0,1)->text());
            float z = atof(ui->BSplinePoints->item(0,2)->text());
            points.push_back(new Point(x, y,z));

            ui->BSplinePoints->removeRow(0);
        }

        std::string name = ui->BSplineName->text().latin1();
        world->createBSplineCurve(name, points, new Color());

        redraw();
    }
}

void gui::on_WireframeAddVertex_clicked()
{
    if(ui->WireframeVertexX->text().compare("")!=0&&ui->WireframeVertexY->text().compare("")!=0&&ui->WireframeVertexZ->text().compare("")!=0){
        ui->WireframeVertices->insertRow(ui->WireframeVertices->rowCount());

        QTableWidgetItem *tmpX = new QTableWidgetItem();
        tmpX->setText(ui->WireframeVertexX->text());
        ui->WireframeVertices->setItem(ui->WireframeVertices->rowCount()-1, 0, tmpX);

        QTableWidgetItem *tmpY = new QTableWidgetItem();
        tmpY->setText(ui->WireframeVertexY->text());
        ui->WireframeVertices->setItem(ui->WireframeVertices->rowCount()-1, 1, tmpY);

        QTableWidgetItem *tmpZ = new QTableWidgetItem();
        tmpZ->setText(ui->WireframeVertexZ->text());
        ui->WireframeVertices->setItem(ui->WireframeVertices->rowCount()-1, 2, tmpZ);

        ui->WireframeVertexX->setText("0");
        ui->WireframeVertexY->setText("0");
        ui->WireframeVertexZ->setText("0");

    }
}

void gui::on_WireframeAddEdge_clicked()
{
    if(ui->WireframeEdgePoint1->text().compare("")!=0&&ui->WireframeEdgePoint2->text().compare("")!=0){
        int v1 = atoi(ui->WireframeEdgePoint1->text());
        int v2 = atoi(ui->WireframeEdgePoint2->text());
        int tam = ui->WireframeVertices->rowCount();
        if (v1 < tam && v2 < tam){
            ui->WireframeEdges->insertRow(ui->WireframeEdges->rowCount());

            QTableWidgetItem *tmp1 = new QTableWidgetItem();
            tmp1->setText(ui->WireframeEdgePoint1->text());
            ui->WireframeEdges->setItem(ui->WireframeEdges->rowCount()-1, 0, tmp1);

            QTableWidgetItem *tmp2 = new QTableWidgetItem();
            tmp2->setText(ui->WireframeEdgePoint2->text());
            ui->WireframeEdges->setItem(ui->WireframeEdges->rowCount()-1, 1, tmp2);

        }
    }
}

void gui::on_WireframeAdd_clicked()
{
    if (ui->WireframeVertices->rowCount() > 1 && ui->WireframeEdges->rowCount() > 0){
        std::list<Point*> vertices;

        while (ui->WireframeVertices->rowCount() > 0) {
            float x = atof(ui->WireframeVertices->item(0,0)->text());
            float y = atof(ui->WireframeVertices->item(0,1)->text());
            float z = atof(ui->WireframeVertices->item(0,2)->text());

            vertices.push_back(new Point(x, y, z));

            ui->WireframeVertices->removeRow(0);
        }
        
        std::list<Edge*> edges;
        std::list<Point*>::iterator it;
        
        while (ui->WireframeEdges->rowCount() > 0) {
            float n1 = atof(ui->WireframeEdges->item(0,0)->text()) - 1;
            float n2 = atof(ui->WireframeEdges->item(0,1)->text()) - 1;

            it = vertices.begin();
            for (int i=0; i<n1; i++)
                it++;
            Point * v1 = *it;
            it = vertices.begin();
            for (int i=0; i<n2; i++)
                it++;
            Point * v2 = *it;

            edges.push_back(new Edge(v1, v2));

            ui->WireframeEdges->removeRow(0);
        }

        std::string name = ui->WireframeName->text().latin1();
        world->createWireframe(name, edges, new Color());

        redraw();
    }
}

void gui::on_CurvedSurfaceAdd_clicked()
{
    Point * pointsMatrix[4][4];

    for (int line = 0; line < 4; line++) {
        for (int column = 0; column < 4; column++) {
            std::string pointString = ui->CurvedSurfacePointsTable->item(line, column)->text().latin1();
            int posStart = 0;
            int posEnd = pointString.find(",", posStart);
            float x = atof(pointString.substr(posStart, posEnd).c_str());
            posStart = posEnd + 1;
            posEnd = pointString.find(",", posStart);
            float y = atof(pointString.substr(posStart, posEnd).c_str());
            posStart = posEnd + 1;
            posEnd = pointString.size();
            float z = atof(pointString.substr(posStart, posEnd).c_str());

            pointsMatrix[line][column] = new Point(x, y, z);
        }
    }

    std::string name = ui->CurvedSurfaceName->text().latin1();
    world->createBSplineSurface(name, pointsMatrix, new Color());

    redraw();
}

void gui::on_actionOpen_triggered()
{
    std::string fileName = QFileDialog::getOpenFileName(this, tr("Open shapes"), "", tr("Shapes file (*.xml)")).latin1();
    XmlParser::openFile(fileName, world, window);
    redraw();
}

void gui::on_actionSave_triggered()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save shapes"), "", tr("Shapes file (*.xml)"));
}
