#ifndef TRANSFORMATIONS_H
#define TRANSFORMATIONS_H

#include "definitions.h"
#include "graphicsobjects/gobject.h"
#include "graphicsobjects/gpoint.h"

using namespace GraphicsObjects;

namespace Transformations
{
    void getTranslationMatrix(Point * translationVector, float * resultMatrix[4][4]);
    void getRotationMatrix(Axis axis, float angle, float resultMatrix[4][4]);
    void getScaleMatrix(Point * anchor, Point * scaleVector, float resultMatrix[4][4]);
    void getTransformationMatrix(Point * translationVector, Point * scaleVector, GPoint * vpn, GPoint * vpu, float resultMatrix[4][4]);
    int translate(GObject * object, float dx, float dy, float dz);
    int translate(GObject * object, Point * axisVector, float distance);
    int rotate(GObject * object, Axis axis, float angle);
    int rotate(GObject * object, Point * anchor, Point * axisVector, float angle);
    int scale(GObject * object, float sx, float sy, float sz);
    int scale(GObject * object, Point * anchor, Point * scaleVector);
    int transform(GObject * object, float matrix[4][4]);
    void transform(Point * point, float matrix[4][4]);
    std::list<GObject*> transform(std::list<GObject*> objects, Point * translationVector, Point * scaleVector, float distanceFromProjectionCenter, GPoint * vpn, GPoint * vpu);

    //Tools
    bool normalize(Point * vector);
    Point * crossProduct(Point * u, Point * v);
    Point * findAlignmentAngles(GPoint * vpn, GPoint * vpu);
    void multiply(float matrixA[4][4], float matrixB[4][4], float resultMatrix[4][4]);
    void multiply(float lineVector[4], float matrix[4][4], float resultVector[4]);
    void multiply(float matrix[4][4], float columnVector[4], float resultVector[4]);
    void transpose(float matrix[4][4], float resultMatrix[4][4]);
    void copy(float originalMatrix[4][4], float copyMatryx[4][4]);
}

#endif // TRANSFORMATIONS_H
