#include "window.h"

Window::Window(World * world, float width, float height, float depth, float distanceFromProjectionCenter)
{
    Window::world = world;
    Window::width = width;
    Window::height = height;
    Window::distanceFromProjectionCenter = distanceFromProjectionCenter;
    Window::depth = depth;
    center = new GPoint(0, 0, 0);
    vpn = new GPoint(0, 0, 1);
    vpu = new GPoint(0, 1, 0);

    //world->createLine("vpn", new Point(0, 0, 0), new Point(1000000, 0, 0), 255, 0, 0);
    //world->createLine("vpu", new Point(0, 0, 0), new Point(0, 1000000, 0), 0, 255, 0);
    //world->createLine("vpx", new Point(0, 0, 0), new Point(0, 0, 1000000), 0, 0, 255);
}

void Window::navigate(Axis axis, float rate) {
    Point * axisVector;
    switch (axis) {
    case X_AXIS:
        axisVector = Transformations::crossProduct(vpn->point(), vpu->point());
        Transformations::translate(center, axisVector, -rate * width);
        //o produto vetorial do vpn com vpu da o eixo x no sentido contrario
        break;
    case Y_AXIS:
        axisVector = vpu->point();
        Transformations::translate(center, axisVector, rate * width);
        break;
    case Z_AXIS:
        axisVector = vpn->point();
        Transformations::translate(center, axisVector, rate * distanceFromProjectionCenter);
        break;
    }

}

void Window::zoom(float rate) {
    width *= (1 - rate);
    height *= (1 - rate);
}

void Window::rotate(Axis axis, float angle) {
    Point * axisVector;
    switch (axis) {
    case X_AXIS:
        axisVector = Transformations::crossProduct(vpn->point(), vpu->point());
        Transformations::rotate(vpn, new Point(0, 0, 0), axisVector, angle);
        Transformations::rotate(vpu, new Point(0, 0, 0), axisVector, angle);
        break;
    case Y_AXIS:
        axisVector = vpu->point();
        Transformations::rotate(vpn, new Point(0, 0, 0), axisVector, angle);
        break;
    case Z_AXIS:
        axisVector = vpn->point();
        Transformations::rotate(vpu, new Point(0, 0, 0), axisVector, angle);
        break;
    }
}

void Window::refresh() {
    objects.clear();
    objects = Transformations::transform(world->objects, new Point(-center->point()->x, -center->point()->y,-center->point()->z), new Point(2/width, 2/height, 2/depth), distanceFromProjectionCenter, vpn, vpu);
}

std::list<GObject*> Window::getViewportObjectsClipped(Point * vpMin, Point * vpMax, float curveAccuracyInViewportDistance){
    refresh();

    float vpHalfWidth = (vpMax->x - vpMin->x)/2;
    float vpHalfHeight = (vpMax->y - vpMin->y)/2;
    float curveAccuracy = curveAccuracyInViewportDistance/(vpHalfWidth + vpHalfHeight) * 2; //normaliza a altura e a largura e faz a mdia
    std::list<GObject*> viewportObjects;
    std::list<GObject*>::iterator objectsIt;

    for (objectsIt = objects.begin(); objectsIt != objects.end(); objectsIt++) {
        GObject * windowObj = (*objectsIt);
        GObject * obj = windowObj->copy();
        std::list<Point*>::iterator pointsIt;

        bool insideViewport;
        if (obj->type().compare("Bezier")==0 || obj->type().compare("B-Spline")==0 || obj->type().compare("B-Spline Surface")==0) {
            insideViewport = static_cast<GCurve*>(obj)->clip(curveAccuracy);
        } else {
            insideViewport = obj->clip();
        }

        if (insideViewport){
            for (pointsIt = obj->points.begin(); pointsIt != obj->points.end(); pointsIt++) {
                (*pointsIt)->x = vpHalfWidth * (*pointsIt)->x + vpMin->x + vpHalfWidth;
                (*pointsIt)->y = -(vpHalfHeight * (*pointsIt)->y + vpHalfHeight) + vpMax->y;
            }
            viewportObjects.push_back(obj);
        }
    }
    return viewportObjects;

}
std::list<GObject*> Window::getViewportObjects(Point * vpMin, Point * vpMax) {
    refresh();
    float vpHalfWidth = (vpMax->x - vpMin->x)/2;
    float vpHalfHeight = (vpMax->y - vpMin->y)/2;
    std::list<GObject*> viewportObjects;
    std::list<GObject*>::iterator objectsIt;
    for (objectsIt = objects.begin(); objectsIt != objects.end(); objectsIt++) {
        GObject * windowObj = (*objectsIt);
        GObject * obj = windowObj->copy();
        std::list<Point*>::iterator pointsIt;
        for (pointsIt = obj->points.begin(); pointsIt != obj->points.end(); pointsIt++) {
            (*pointsIt)->x = vpHalfWidth * (*pointsIt)->x + vpMin->x + vpHalfWidth;
            (*pointsIt)->y = -(vpHalfHeight * (*pointsIt)->y + vpHalfHeight) + vpMax->y;
        }
        viewportObjects.push_back(obj);
    }

    return viewportObjects;
}

void Window::setAtributes(Point * cop, Point * vpn, Point * vpu, Point * wmin, Point * wmax) {
    width = wmax->x - wmin->x;
    height = wmax->y - wmin->y;
    depth = wmax->z - wmin->z;
    float cx = width/2 + wmin->x;
    float cy = height/2 + wmin->y;
    float cz = depth/2 + wmin->z;
    center = new GPoint(cx, cy, cz);
    Window::vpn = new GPoint(vpn->x, vpn->y, vpn->z);
    Window::vpu = new GPoint(vpu->x, vpu->y, vpu->z);
    distanceFromProjectionCenter = cop->distance(center->point());
}
