#include "world.h"

//Private

std::string World::generateUniqueName(std::string name) {
    if (objectExists(name)) {
        name += "1";
    }
    return name;
}

//Public

World::World()
{
        DBG1("object world successfully created");
}

World::~World(){
        DBG1("object world successfully deleted");
}

GPoint * World::createPoint(std::string name, Point * point, Color * color) {
    name = generateUniqueName(name);
    GPoint * gpoint = new GPoint(name, point, color);
    objects.push_back(gpoint);
    return gpoint;
}

GLine * World::createLine(std::string name, Point * p1, Point * p2, Color * color) {
    name = generateUniqueName(name);
    GLine * gline = new GLine(name, p1, p2, color);
    objects.push_back(gline);
    return gline;
}

GPolygon * World::createPolygon(std::string name, std::list<Point*> points, Color * color, bool fill) {
    if (points.size() < 2) {
        return NULL;
    }

    name = generateUniqueName(name);
    GPolygon * gpolign = new GPolygon(name, points, color, fill);
    objects.push_back(gpolign);
    return gpolign;
}

GBezier * World::createBezierCurve(std::string name, std::list<Point*> points, Color * color) {
    if (points.size() < 4) {
        return NULL;
    }

    name = generateUniqueName(name);
    GBezier * gbezier = new GBezier(name, points, color);
    objects.push_back(gbezier);
    return gbezier;
}

GBSpline * World::createBSplineCurve(std::string name, std::list<Point*> points, Color * color) {
    if (points.size() < 4) {
        return NULL;
    }

    name = generateUniqueName(name);
    GBSpline * gbspline = new GBSpline(name, points, color);
    objects.push_back(gbspline);
    return gbspline;
}

GWireframe * World::createWireframe(std::string name, std::list<Edge*> edges, Color * color) {
    if (edges.size() < 1) {
        return NULL;
    }

    name = generateUniqueName(name);
    GWireframe * gwireframe = new GWireframe(name, edges, color);
    objects.push_back(gwireframe);
    return gwireframe;
}

GBSplineSurface * World::createBSplineSurface(std::string name, Point * pointsMatrix[4][4], Color * color) {
    name = generateUniqueName(name);
    GBSplineSurface * gbsplinesurface = new GBSplineSurface(name, pointsMatrix, color);
    objects.push_back(gbsplinesurface);
    return gbsplinesurface;
}

bool World::objectExists(std::string name){
    GObject * obj = getObject(name);
    if (obj == NULL)
        return false;
    return true;
}

int World::translate(std::string name, float dx, float dy, float dz){
    GObject * obj = getObject(name);
    if(obj==NULL)
        return ERROR_OBJECT_NOT_EXIST;
    return Transformations::translate(obj, dx, dy, dz);
}

int World::rotateAroundCenter(std::string name, Point * axisVector, float angle){
    GObject * obj = getObject(name);
    if(obj==NULL)
        return ERROR_OBJECT_NOT_EXIST;
    return Transformations::rotate(obj, obj->center(), axisVector, angle);
}

int World::rotateAroundOrigin(std::string name, Point * axisVector, float angle){
    GObject * obj = getObject(name);
    if(obj==NULL)
        return ERROR_OBJECT_NOT_EXIST;
    return Transformations::rotate(obj, new Point(0, 0, 0), axisVector, angle);
}

int World::rotate(std::string name, Point * anchor, Point * axisVector, float angle){
    GObject * obj = getObject(name);
    if(obj==NULL)
        return ERROR_OBJECT_NOT_EXIST;
    return Transformations::rotate(obj, anchor, axisVector, angle);
}

int World::scale(std::string name, float sx, float sy, float sz){
    GObject * obj = getObject(name);
    if(obj==NULL)
        return ERROR_OBJECT_NOT_EXIST;
    return Transformations::scale(obj, obj->center(), new Point(sx, sy, sz));
}

std::list<std::string> World::objectsStringsList(){
    std::list<std::string> ret;
    std::list<GObject* >::iterator it;
    for (it = objects.begin(); it != objects.end(); it++){
        ret.push_back((*it)->toString());
    }

    return ret;
};

GObject* World::getObject(std::string name){
    GObject * ret = NULL;
    std::list<GObject* >::iterator it;
    for (it = objects.begin(); it != objects.end(); it++){
        if (name.compare((*it)->name) == 0)
            ret = (*it);
    }

    return ret;
};
