#ifndef WORLD_H
#define WORLD_H

#include "definitions.h"
#include "transformations.h"
#include "graphicsobjects/gobject.h"
#include "graphicsobjects/gpoint.h"
#include "graphicsobjects/gline.h"
#include "graphicsobjects/gpolygon.h"
#include "graphicsobjects/gbezier.h"
#include "graphicsobjects/gbspline.h"
#include "graphicsobjects/gwireframe.h"
#include "graphicsobjects/gbsplinesurface.h"

using namespace GraphicsObjects;

class World
{
    std::string generateUniqueName(std::string name);
public:
    World();
    ~World();
    std::list<GObject*> objects;

    GPoint * createPoint(std::string name, Point * point, Color * color);
    GLine * createLine(std::string name, Point * p1, Point * p2, Color * color);
    GPolygon * createPolygon(std::string name, std::list<Point*> points, Color * color, bool fill);
    GBezier * createBezierCurve(std::string name, std::list<Point*> points, Color * color);
    GBSpline * createBSplineCurve(std::string name, std::list<Point*> points, Color * color);
    GWireframe * createWireframe(std::string name, std::list<Edge*> edges, Color * color);
    GBSplineSurface * createBSplineSurface(std::string name, Point * pointsMatrix[4][4], Color * color);
    bool objectExists(std::string name);
    int translate(std::string name, float dx, float dy, float dz);
    int rotateAroundCenter(std::string name, Point * axisVector, float angle);
    int rotateAroundOrigin(std::string name, Point * axisVector, float angle);
    int rotate(std::string name, Point * anchor, Point * axisVector, float angle);
    int scale(std::string name, float sx, float sy, float sz);
    int transform(std::string name, float matrix[3][3]);
    std::list<std::string> objectsStringsList();

    GObject* getObject(std::string name);

};

#endif // WORLD_H
