#include "xmlparser.h"

namespace XmlParser {

    void openFile(std::string fileName, World * world, Window * window) {
        QXmlQuery masterQuery;
        std::string masterXPath = "doc('" + fileName + "')//formas" + "|doc('" + fileName + "')";
        masterQuery.setQuery(masterXPath.c_str());
        std::cout << masterXPath.c_str() << std::endl;

        QXmlResultItems masterResult;
        masterQuery.evaluateTo(&masterResult);
        QXmlItem masterItem(masterResult.next());
        while (!masterItem.isNull()) {

            masterQuery.setFocus(masterItem);
            QXmlResultItems resultShapes;
            QXmlItem shapeItem;

            masterQuery.setQuery("./window");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "Window loaded" << std::endl;
                setWindowAttributes(masterQuery, &shapeItem, window);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./ponto");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "Point loaded" << std::endl;
                constructPoint(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./reta");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "Line loaded" << std::endl;
                constructLine(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./poligono");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "Polygon loaded" << std::endl;
                constructPolygon(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./curvaBSpline");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "B-Spline Curve loaded" << std::endl;
                constructBSplineCurve(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./curvaBezier");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "Bezier Curve loaded" << std::endl;
                constructBezierCurve(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./objeto3d");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "Wireframe loaded" << std::endl;
                constructWireframe(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterQuery.setQuery("./superficieBSpline");
            masterQuery.evaluateTo(&resultShapes);
            shapeItem = resultShapes.next();
            while (!shapeItem.isNull()) {
                std::cout << "B-Spline Surface loaded" << std::endl;
                constructBSplineSurface(masterQuery, &shapeItem, world);
                shapeItem = resultShapes.next();
            }

            masterItem = masterResult.next();
        }
    }

    void setWindowAttributes(QXmlQuery query, QXmlItem * shapeItem, Window * window) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultAttributes;

        query.setQuery("./cop");
        query.evaluateTo(&resultAttributes);
        Point * cop = constructPoint(query, &resultAttributes.next());

        query.setQuery("./vpn");
        query.evaluateTo(&resultAttributes);
        Point * vpn = constructPoint(query, &resultAttributes.next());

        query.setQuery("./vup");
        query.evaluateTo(&resultAttributes);
        Point * vpu = constructPoint(query, &resultAttributes.next());

        query.setQuery("./wmin");
        query.evaluateTo(&resultAttributes);
        Point * wmin = constructPoint(query, &resultAttributes.next());

        query.setQuery("./wmax");
        query.evaluateTo(&resultAttributes);
        Point * wmax = constructPoint(query, &resultAttributes.next());

        window->setAtributes(cop, vpn, vpu, wmin, wmax);
    }

    Point * constructPoint(QXmlQuery query, QXmlItem * shapeItem) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultAttributes;

        query.setQuery("./@x/string()");
        query.evaluateTo(&resultAttributes);
        float x = resultAttributes.next().toAtomicValue().toFloat();

        query.setQuery("./@y/string()");
        query.evaluateTo(&resultAttributes);
        float y = resultAttributes.next().toAtomicValue().toFloat();

        query.setQuery("./@z/string()");
        query.evaluateTo(&resultAttributes);
        float z = resultAttributes.next().toAtomicValue().toFloat();

        return new Point(x, y, z);
    }

    Edge * constructEdge(QXmlQuery query, QXmlItem * shapeItem, std::list<Point*> vertices) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultAttributes;

        query.setQuery("./@v1/string()");
        query.evaluateTo(&resultAttributes);
        int n1 = resultAttributes.next().toAtomicValue().toInt();

        query.setQuery("./@v2/string()");
        query.evaluateTo(&resultAttributes);
        int n2 = resultAttributes.next().toAtomicValue().toInt();

        std::list<Point*>::iterator it = vertices.begin();
        for (int i=0; i<n1; i++)
            it++;
        Point * v1 = *it;
        it = vertices.begin();
        for (int i=0; i<n2; i++)
            it++;
        Point * v2 = *it;

        return new Edge(v1, v2);
    }

    GPoint * constructPoint(QXmlQuery query, QXmlItem * shapeItem, World * world) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        Point * p = constructPoint(query, shapeItem);

        return world->createPoint(name, p, new Color(colorStr));
    }

    GLine * constructLine(QXmlQuery query, QXmlItem * shapeItem, World * world) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./ponto");
        query.evaluateTo(&resultItems);
        Point * p1 = constructPoint(query, &resultItems.next());
        Point * p2 = constructPoint(query, &resultItems.next());

        return world->createLine(name, p1, p2, new Color(colorStr));
    }

    GPolygon * constructPolygon(QXmlQuery query, QXmlItem * shapeItem, World * world) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cheio/string()");
        query.evaluateTo(&resultItems);
        bool fill = resultItems.next().toAtomicValue().toBool();

        std::list<Point*> points;
        query.setQuery("./ponto");
        query.evaluateTo(&resultItems);
        QXmlItem pointItem = resultItems.next();
        while (!pointItem.isNull()) {
            points.push_back(constructPoint(query, &pointItem));

            pointItem = resultItems.next();
        }

        return world->createPolygon(name, points, new Color(colorStr), fill);
    }

    GBSpline * constructBSplineCurve(QXmlQuery query, QXmlItem * shapeItem, World * world) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        std::list<Point*> points;
        query.setQuery("./ponto");
        query.evaluateTo(&resultItems);
        QXmlItem pointItem = resultItems.next();
        while (!pointItem.isNull()) {
            points.push_back(constructPoint(query, &pointItem));

            pointItem = resultItems.next();
        }

        return world->createBSplineCurve(name, points, new Color(colorStr));
    }

    GBezier * constructBezierCurve(QXmlQuery query, QXmlItem * shapeItem, World * world)  {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        std::list<Point*> points;
        query.setQuery("./geometria/*");
        query.evaluateTo(&resultItems);
        QXmlItem pointItem = resultItems.next();
        while (!pointItem.isNull()) {
            points.push_back(constructPoint(query, &pointItem));

            pointItem = resultItems.next();
        }

        return world->createBezierCurve(name, points, new Color(colorStr));
    }

    GWireframe * constructWireframe(QXmlQuery query, QXmlItem * shapeItem, World * world) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        std::list<Point*> points;
        query.setQuery("./pontos/ponto");
        query.evaluateTo(&resultItems);
        QXmlItem pointItem = resultItems.next();
        while (!pointItem.isNull()) {
            points.push_back(constructPoint(query, &pointItem));

            pointItem = resultItems.next();
        }

        std::list<Edge*> edges;
        query.setQuery("./arestas/aresta");
        query.evaluateTo(&resultItems);
        QXmlItem edgeItem = resultItems.next();
        while (!edgeItem.isNull()) {
            edges.push_back(constructEdge(query, &edgeItem, points));

            edgeItem = resultItems.next();
        }

        return world->createWireframe(name, edges, new Color(colorStr));
    }

    GBSplineSurface * constructBSplineSurface(QXmlQuery query, QXmlItem * shapeItem, World * world) {
        query.setFocus(*shapeItem);
        QXmlResultItems resultItems;

        query.setQuery("./@nome/string()");
        query.evaluateTo(&resultItems);
        std::string name = resultItems.next().toAtomicValue().toString().latin1();

        query.setQuery("./@cor/string()");
        query.evaluateTo(&resultItems);
        std::string colorStr = resultItems.next().toAtomicValue().toString().latin1();

        Point * points[4][4];
        query.setQuery("./geometria/elemento");
        query.evaluateTo(&resultItems);
        QXmlItem pointItem = resultItems.next();
        int line = 0;
        int column = 0;
        while (!pointItem.isNull()) {
            points[line][column] = constructPoint(query, &pointItem);

            pointItem = resultItems.next();
            column ++;
            if (column == 4) {
                column = 0;
                line ++;
                if (line == 4) {
                    break;
                }
            }
        }

        return world->createBSplineSurface(name, points, new Color(colorStr));
    }

}
